#!/usr/bin/perl
# ----------------------------------------------------------------- #
#           The HMM-Based Speech Synthesis System (HTS)             #
#           developed by HTS Working Group                          #
#           http://hts.sp.nitech.ac.jp/                             #
# ----------------------------------------------------------------- #
#                                                                   #
#  Copyright (c) 2001-2009  Nagoya Institute of Technology          #
#                           Department of Computer Science          #
#                                                                   #
# All rights reserved.                                              #
#                                                                   #
# Redistribution and use in source and binary forms, with or        #
# without modification, are permitted provided that the following   #
# conditions are met:                                               #
#                                                                   #
# - Redistributions of source code must retain the above copyright  #
#   notice, this list of conditions and the following disclaimer.   #
# - Redistributions in binary form must reproduce the above         #
#   copyright notice, this list of conditions and the following     #
#   disclaimer in the documentation and/or other materials provided #
#   with the distribution.                                          #
# - Neither the name of the HTS working group nor the names of its  #
#   contributors may be used to endorse or promote products derived #
#   from this software without specific prior written permission.   #
#                                                                   #
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND            #
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,       #
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF          #
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE          #
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS #
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,          #
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED   #
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     #
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON #
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   #
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY    #
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE           #
# POSSIBILITY OF SUCH DAMAGE.                                       #
# ----------------------------------------------------------------- #

if (@ARGV != 2) {
  print "\nUsage: perl makelabel.pl text_proc_output_file HTS_label_file\n\n";
  exit (0);
}
($textfile, $htsfile) = @ARGV;

# open input file
open(TXT, "@ARGV[0]");
@UTT = <TXT>;
close TXT;
$nlines = @UTT;

# content word definitions
@content_words = ("ver", "subs", "adj", "adv");
@vowels = ("a", "a~", "E", "e", "e~", "i", "i~", "O", "o", "o~", "u", "u~");

# inicialize counters
$total_mono = 1;
$total_syll = 1;
$total_wor = 1;
$total_phr = 0;
$total_sent = 1;
$number_phr_per_sent[$total_sent] = 1;
$phrase_flag = 0;



#------------- process each line of the utterance file to collect information

for ($l = 1; $l < @UTT; $l++)
{
        $line = $UTT[$l];
        chomp($line);
	@line_info = split " ",$line;
	$nline_info = @line_info;
	
	$monophone[$total_mono] = $line_info[0];
	
        if ($nline_info == 1)
        {
                if ( $monophone[$total_mono] eq "sil" )
                {
                        $monoposition[$total_mono] = "x";
                        $word_flag = 0;
                        if ( $phrase_flag == 0)
                        {
                        	$total_phr++;
                                $phrposition[$total_phr] = 1;
                                $phrase_flag = 1;
			}
		}
		elsif ( ($monophone[$total_mono] eq "lp") || ($monophone[$total_mono] eq "pau") )
		{
			$monoposition[$total_mono] = "x";
                        $word_flag = 0;
                        $total_phr++;
                        $phrposition[$total_phr] = $phrposition[$total_phr-1] + 1;
                        $number_phr_per_sent[$total_sent]++;
		}
                else
                {
                        $monoposition[$total_mono] = $monoposition[$total_mono-1] + 1;
                }
                $total_mono++;
	}
        else
        {
                $monoposition[$total_mono] = 1;
                $total_mono++;
		$syllable[$total_syll] = $line_info[1];
				
		for ($j = 0; $j < @vowels; $j++)
		{
			$i = index $syllable[$total_syll],$vowels[$j];
		        if ($i >= 0)
			{
			        $vowel_syllable[$total_syll] = substr $syllable[$total_syll],$i,length($vowels[$j]);
			}
		}
		if ( ($monophone[$total_mono-2] eq "pau") || ($monophone[$total_mono-2] eq "lp") || ($monophone[$total_mono-2] eq "sil") )
		{
		        $syllposition_phr[$total_syll] = 1;
		}
		else
		{
		        $syllposition_phr[$total_syll] = $syllposition_phr[$total_syll-1] + 1;
		}
				        
                $stressflag[$total_syll] = $line_info[2];
                                
                if ($nline_info == 3)
                {
                        $syllposition[$total_syll] = $syllposition[$total_syll-1] + 1;
                        $total_syll++;
		}
                else
                {
                        $syllposition[$total_syll] = 1;
                        $total_syll++;
                        $wordclass[$total_wor] = $line_info[4];
                                        
                        if ($nline_info == 5)
                        {
                                $interrflag[$total_wor] = 0;
			}
                        else
                        {
                                $interrflag[$total_wor] = $line_info[5];
			}
                        if ($word_flag == 0)
                        {
                        	$worposition[$total_wor] = 1;
                        }
                        else
                        {
                        	$worposition[$total_wor] = $worposition[$total_wor-1] + 1;
                        }
                        $word_flag = 1;
                        $total_wor++;
                        $number_wor_per_phr[$total_phr]++;
                        $number_wor_per_sent[$total_sent]++;
                        if ( ($wordclass[$total_wor-1] eq $content_words[0]) || ($wordclass[$total_wor-1] eq $content_words[1]) || ($wordclass[$total_wor-1] eq $content_words[2]) || ($wordclass[$total_wor-1] eq $content_words[3]) )
                        {
                        	$number_content_per_phr[$total_phr]++;
                                $number_content_per_sent[$total_sent]++;
			}
                        else
                        {
                        	$number_content_per_phr[$total_phr] += 0;
                                $number_content_per_sent[$total_sent] += 0;
			}
		}
                $number_syll_per_phr[$total_phr]++;
                $number_syll_per_sent[$total_sent]++;
                if ( $stressflag[$total_syll-1] == 1 )
                {
                	$number_stressed_per_phr[$total_phr]++;
                        $number_stressed_per_sent[$total_sent]++;
		}
                else
                {
                	$number_stressed_per_phr[$total_phr] += 0;
                        $number_stressed_per_sent[$total_sent] += 0;
		}
	}
}



#--------- processing syllable information --------------

# number of phonemes / syllable
$k = 0;
for ($j = 1; $j < $total_mono; $j++)
{
        if ($monoposition[$j] == 1)
        {
                $k++;
                $numberphon[$k] = 1;
        }
        elsif ($monoposition[$j] ne "x")
        {
                $numberphon[$k]++;
        }
}

# number of stressed syllables before and after in each phrase
$aux = 0;
for ($j = 1; $j <= $total_phr; $j++)
{
        if ($stressflag[1 + $aux] == 0)
        {
                $stress_before[1 + $aux] = 0;
                $stress_after[1 + $aux] = $number_stressed_per_phr[$j];
                $aux_flag = 0;
        }
        else
        {
                $stress_before[1 + $aux] = 0;
                $stress_after[1 + $aux] = $number_stressed_per_phr[$j] - 1;
                $aux_flag = 1;
        }
        for ( $k =2 + $aux; $k <= $number_syll_per_phr[$j] + $aux; $k++ )
        {
                if ($aux_flag == 0)
                {
                        $stress_before[$k] = $stress_before[$k - 1];
                }
                else
                {
                        $stress_before[$k] = $stress_before[$k - 1] + 1;
                }
                if ($stressflag[$k] == 0)
                {
                        $stress_after[$k] = $stress_after[$k - 1];
                        $aux_flag = 0;
                }
                else
                {
                        $stress_after[$k] = $stress_after[$k - 1] - 1;
                        $aux_flag = 1;
                }
        }
        $aux = $aux + $number_syll_per_phr[$j];
}

# number of stressed syllables from previous stressed to current (1), and from current to next stressed (2)
# for each sentence
$aux = 0;
for ($l = 1; $l <= $total_sent; $l++)
{
        for ($k = 1 + $aux; $k <= $number_syll_per_sent[$l] + $aux; $k++)
        {
                $j = $k-1;
                while ( ($stressflag[$j] == 0) && ($j > $aux) )
                {
                        $j--;
                }
                $i = $k + 1;
                while ( ($stressflag[$i] == 0) && ($i <= $number_syll_per_sent[$l] + $aux) )
                {
                        $i++;
                }
                if ( ($j == $aux) && ($i > $number_syll_per_sent[$l] + $aux) )
                {
                        $between_stressed1[$k] = 0;
                        $between_stressed2[$k] = 0;
                }
                elsif ( ($j == $aux) && ($i <= $number_syll_per_sent[$l] + $aux) )
                {
                        $between_stressed1[$k] = 0;
                        $between_stressed2[$k] = $i - $k + 1;
                }
                elsif ( ($j > $aux) && ($i > $number_syll_per_sent[$l] + $aux) )
                {
                        $between_stressed1[$k] = $k - $j + 1;
                        $between_stressed2[$k] = 0;
                }
                elsif ( ($j > $aux) && ($i <= $number_syll_per_sent[$l] + $aux) )
                {
                        $between_stressed1[$k] = $k - $j + 1;
                        $between_stressed2[$k] = $i - $k + 1;
                }
                else
                {
                        print SDTOUT "Error: this option was not supposed to be possible.\n"
                }
        }
        $aux = $aux + $number_syll_per_sent[$l];
}



#--------- processing word information --------------

# number of syllable / word
$k = 0;
for ($j = 1; $j < $total_syll; $j++)
{
        if ($syllposition[$j] == 1)
        {
                $k++;
                $numbersyll[$k] = 1;
        }
        else
        {
                $numbersyll[$k]++;
        }
}

# number of content words before and after in each phrase
$aux = 0;
for ($j = 1; $j <= $total_phr; $j++)
{
        for ( $k =1 + $aux; $k <= $number_wor_per_phr[$j] + $aux; $k++ )
        {
                $wrdclass_before[$k] = 0;
                for ($i = 1 + $aux; $i < $k; $i++)
                {
                        if ( ($wordclass[$i] eq $content_words[0]) || ($wordclass[$i] eq $content_words[1]) || ($wordclass[$i] eq $content_words[2]) || ($wordclass[$i] eq $content_words[3]) )
                        {
                                $wrdclass_before[$k]++;
                        }
                }
                $wrdclass_after[$k] = 0;
                for ($i = $k + 1; $i <= $number_wor_per_phr[$j] + $aux; $i++)
                {
                        if ( ($wordclass[$i] eq $content_words[0]) || ($wordclass[$i] eq $content_words[1]) || ($wordclass[$i] eq $content_words[2]) || ($wordclass[$i] eq $content_words[3]) )
                        {
                                $wrdclass_after[$k]++;
                        }
                }
        }
        $aux += $number_wor_per_phr[$j];
}

# number of content words from previous to current and from current to next
# for each sentence
$aux = 0;
for ($l = 1; $l <= $total_sent; $l++)
{
        for ( $k =1 + $aux; $k <= $number_wor_per_sent[$l] + $aux; $k++ )
        {
                $i = $k - 1;
                while ( ( ($wordclass[$i] ne $content_words[0]) && ($wordclass[$i] ne $content_words[1]) && ($wordclass[$i] ne $content_words[2]) && ($wordclass[$i] ne $content_words[3]) ) && ($i > $aux) )
                {
                        $i--;
                }
                $j = $k + 1;
                while ( ( ($wordclass[$j] ne $content_words[0]) && ($wordclass[$j] ne $content_words[1]) && ($wordclass[$j] ne $content_words[2]) && ($wordclass[$j] ne $content_words[3]) ) && ($j <= $number_wor_per_sent[$l] + $aux) )
                {
                        $j++;
                }
                if ( ($i == $aux) && ($j > $number_wor_per_sent[$l] + $aux) )
                {
                        $between_wrdclass1[$k] = 0;
                        $between_wrdclass2[$k] = 0;
                }
                elsif ( ($i == $aux) && ($j <= $number_wor_per_sent[$l] + $aux) )
                {
                        $between_wrdclass1[$k] = 0;
                        $between_wrdclass2[$k] = $j - $k + 1;
                }
                elsif ( ($i > $aux) && ($j > $number_wor_per_sent[$l] + $aux) )
                {
                        $between_wrdclass1[$k] = $k - $i + 1;
                        $between_wrdclass2[$k] = 0;
                }
                elsif ( ($i > $aux) && ($j <= $number_wor_per_sent[$l] + $aux) )
                {
                        $between_wrdclass1[$k] = $k - $i + 1;
                        $between_wrdclass2[$k] = $j - $k + 1;
                }
                else
                {
                        print SDTOUT "Error: this option was supposed to be not possible.\n"
                }
        }
        $aux = $aux + $number_wor_per_sent[$l];
}



#--------------- Write information in the output full context label ----------------------

open(HTS, ">@ARGV[1]");

$k = 1;
$i = 1;
$p = 1;
$l = 1;

$aux3 = 0;
$aux4 = 0;
$aux5 = 0;

for ($j = 0; $j < $total_wor; $j++)
{
	if ( ($wordclass[$j] ne "content") && ($wordclass[$j] ne "function") )
	{
		if ( ($wordclass[$j] eq $content_words[0]) || ($wordclass[$j] eq $content_words[1]) || ($wordclass[$j] eq $content_words[2]) || ($wordclass[$j] eq $content_words[3]) )
		{
                	$wordclass[$j] = "content";
        	}
        	else
        	{
                	$wordclass[$j] = "function";
        	}
	}
 }
for ($j = 1; $j < $total_mono; $j++)
{
        if ($monophone[$j] eq "sil")
        {
                if ( $j - 2 < 1 )
		{
		        print HTS "y^";
		}
		else
		{
		        print HTS "$monophone[$j-2]^";
		}
		if ( $j - 1 < 1 )
		{
		        print HTS "y-$monophone[$j]+";
		}
		else
		{
		        print HTS "$monophone[$j-1]-$monophone[$j]+";
		}
                if ( $j + 1 > $total_mono - 1 )
		{
		        print HTS "y=";
		}
		else
		{
		        print HTS "$monophone[$j+1]=";
		}
                if ( $j + 2 > $total_mono - 1 )
		{
		        print HTS "y/M2:y\_y/U:";
		}
		else
		{
		        print HTS "$monophone[$j+2]/M2:y\_y/U:";
		}
                print HTS "$number_syll_per_sent[$l]\_\$$number_wor_per_sent[$l]\_\&$number_phr_per_sent[$l]\n";
        }
        elsif ( ($monophone[$j] eq "lp") || ($monophone[$j] eq "pau") )
        {
		if ( ( ($monophone[$j-1] eq "sil") || ($j-2<1) ) && ( ($monophone[$j+1] eq "sil") || ($j+3>$total_mono) ) )
		{
		        print HTS "y^$monophone[$j-1]-$monophone[$j]+$monophone[$j+1]=y/M2:y\_y/S1:";
		}
		elsif ( ($monophone[$j-1] eq "sil")  || ($j-2<1) )
		{
		        print HTS "y^$monophone[$j-1]-$monophone[$j]+$monophone[$j+1]=$monophone[$j+2]/M2:y\_y/S1:";
		}
		elsif ( ($monophone[$j+1] eq "sil") || ($j+3>$total_mono) )
		{
		        print HTS "$monophone[$j-2]^$monophone[$j-1]-$monophone[$j]+$monophone[$j+1]=y/M2:y\_y/S1:";
		}
		else
		{
		        print HTS "$monophone[$j-2]^$monophone[$j-1]-$monophone[$j]+$monophone[$j+1]=$monophone[$j+2]/M2:y\_y/S1:";
		}
		
                print HTS "$stressflag[$k-1]\_\@$numberphon[$k-1]-y\_\@y+$stressflag[$k]\_\@$numberphon[$k]/S2:y\_y/S3:y\_y/S4:";
                print HTS "$number_stressed_per_phr[$p-1]\_$number_stressed_per_phr[$p]/S5:$between_stressed1[$k]_$between_stressed2[$k]/S6:y/W1:";
                print HTS "$wordclass[$i-1]\_\#$numbersyll[$i-1]-y\_\#y+$wordclass[$i]\_\#$numbersyll[$i]/W2:y\_y/W3:";
                print HTS "$number_content_per_phr[$p-1]\_$number_content_per_phr[$p]/W4:$between_wrdclass1[$i]\_$between_wrdclass2[$i]/P1:";
                print HTS "$number_syll_per_phr[$p-1]\_\!$number_wor_per_phr[$p-1]-y\_\!y+$number_syll_per_phr[$p]\_\!$number_wor_per_phr[$p]/P2:y\_\!y/U:";
                print HTS "$number_syll_per_sent[$l]\_\$$number_wor_per_sent[$l]\_\&$number_phr_per_sent[$l]\n";
        }
        else
        {
                $back_position = $numberphon[$k] - $monoposition[$j] + 1;
		if ( ( ($monophone[$j-1] eq "sil") || ($j-2<1) ) && ( ($monophone[$j+1] eq "sil") || ($j+3>$total_mono) ) )
		{
                        print HTS "y^$monophone[$j-1]-$monophone[$j]+$monophone[$j+1]=y/M2:$monoposition[$j]\_$back_position/S1:";
		}
		elsif ( ($monophone[$j-1] eq "sil")  || ($j-2<1) )
		{
                        print HTS "y^$monophone[$j-1]-$monophone[$j]+$monophone[$j+1]=$monophone[$j+2]/M2:$monoposition[$j]\_$back_position/S1:";
		}
		elsif ( ($monophone[$j+1] eq "sil") || ($j+3>$total_mono) )
		{
                        print HTS "$monophone[$j-2]^$monophone[$j-1]-$monophone[$j]+$monophone[$j+1]=y/M2:$monoposition[$j]\_$back_position/S1:";
                }
		else
		{
                        print HTS "$monophone[$j-2]^$monophone[$j-1]-$monophone[$j]+$monophone[$j+1]=$monophone[$j+2]/M2:$monoposition[$j]\_$back_position/S1:";
                }
		
                $back_position = $numbersyll[$i] - $syllposition[$k] + 1;
                if ( ($syllposition[$k] == 1) && ($phrposition[$p] == 1) && ($k == $number_syll_per_sent[$l] + $aux3) )
                {
                        print HTS "y\_\@y-$stressflag[$k]\_\@$numberphon[$k]+y\_\@y/S2:$syllposition[$k]\_$back_position/S3:";
                }
                elsif ( ($syllposition[$k] == 1) && ($phrposition[$p] == 1) )
                {
                        print HTS "y\_\@y-$stressflag[$k]\_\@$numberphon[$k]+$stressflag[$k+1]\_\@$numberphon[$k+1]/S2:$syllposition[$k]\_$back_position/S3:";
                }
                elsif ( $k == $number_syll_per_sent[$l] + $aux3 )
                {
                        print HTS "$stressflag[$k-1]\_\@$numberphon[$k-1]-$stressflag[$k]\_\@$numberphon[$k]+y\_\@y/S2:$syllposition[$k]\_$back_position/S3:";
                }
                else
                {
                        print HTS "$stressflag[$k-1]\_\@$numberphon[$k-1]-$stressflag[$k]\_\@$numberphon[$k]+$stressflag[$k+1]\_\@$numberphon[$k+1]/S2:$syllposition[$k]\_$back_position/S3:";
                }
                
		$back_position = $number_syll_per_phr[$p] - $syllposition_phr[$k] + 1;
                print HTS "$syllposition_phr[$k]\_$back_position/S4:";
                print HTS "$stress_before[$k]\_$stress_after[$k]/S5:$between_stressed1[$k]\_$between_stressed2[$k]/S6:$vowel_syllable[$k]/W1:";
                $back_position = $number_wor_per_phr[$p] - $worposition[$i] + 1;
                if ( ($worposition[$i] == 1) && ($phrposition[$p] == 1) && ($i == $number_wor_per_sent[$l] + $aux4) )
                {
                        print HTS "y\_\#y-$wordclass[$i]\_\#$numbersyll[$i]+y\_\#y/W2:$worposition[$i]\_$back_position/W3:";
                }
                elsif ( ($worposition[$i] == 1) && ($phrposition[$p] == 1) )
                {
                        print HTS "y\_\#y-$wordclass[$i]\_\#$numbersyll[$i]+$wordclass[$i+1]\_\#$numbersyll[$i+1]/W2:$worposition[$i]\_$back_position/W3:";
                }
                elsif ( $i == $number_wor_per_sent[$l] + $aux4 )
                {
                        print HTS "$wordclass[$i-1]\_\#$numbersyll[$i-1]-$wordclass[$i]\_\#$numbersyll[$i]+y\_\#y/W2:$worposition[$i]\_$back_position/W3:";
                }
                else
                {
                        print HTS "$wordclass[$i-1]\_\#$numbersyll[$i-1]-$wordclass[$i]\_\#$numbersyll[$i]+$wordclass[$i+1]\_\#$numbersyll[$i+1]/W2:$worposition[$i]\_$back_position/W3:";
                }
                print HTS "$wrdclass_before[$i]\_$wrdclass_after[$i]/W4:$between_wrdclass1[$i]\_$between_wrdclass2[$i]/P1:";
                $back_position = $number_phr_per_sent[$l] - $phrposition[$p] + 1;
                if ( ($phrposition[$p] == 1) && ( $p == $number_phr_per_sent[$l] + $aux5) )
                {
                        print HTS "y\_\!y-$number_syll_per_phr[$p]\_\!$number_wor_per_phr[$p]+y\_\!y/P2:$phrposition[$p]\_$back_position/U:";
                }
                elsif ( $phrposition[$p] == 1 )
                {
                        print HTS "y\_\!y-$number_syll_per_phr[$p]\_\!$number_wor_per_phr[$p]+$number_syll_per_phr[$p+1]\_\!$number_wor_per_phr[$p+1]/P2:$phrposition[$p]\_$back_position/U:";
                }
                elsif ( $p == $number_phr_per_sent[$l] + $aux5)
                {
                        print HTS "$number_syll_per_phr[$p-1]\_\!$number_wor_per_phr[$p-1]-$number_syll_per_phr[$p]\_\!$number_wor_per_phr[$p]+y\_\!y/P2:$phrposition[$p]\_$back_position/U:";
                }
                else
                {
                        print HTS "$number_syll_per_phr[$p-1]\_\!$number_wor_per_phr[$p-1]-$number_syll_per_phr[$p]\_\!$number_wor_per_phr[$p]+$number_syll_per_phr[$p+1]\_\!$number_wor_per_phr[$p+1]/P2:$phrposition[$p]\_$back_position/U:";
                }
                print HTS "$number_syll_per_sent[$l]\_\$$number_wor_per_sent[$l]\_\&$number_phr_per_sent[$l]\n";
                if ( ($monoposition[$j+1] == 1) || ($monoposition[$j+1] eq "x") )
                {
                        $k++;
                        if ( $syllposition[$k] == 1 )
                        {
                                $i++;
                                if ( $worposition[$i] == 1 )
                                {
                                        $p++;
                                        if ($phrposition[$p] == 1 )
                                        {
                                                $aux3 += $number_syll_per_sent[$l];
                                                $aux4 += $number_wor_per_sent[$l];
                                                $aux5 += $number_phr_per_sent[$l];
                                                $l++;
                                        }
                                }
                        }
                }
        }
}
close(HTS);
